# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/grokking',
                'doc_host': 'https://smithjessk.github.io',
                'git_url': 'https://github.com/smithjessk/grokking',
                'lib_path': 'grokking'},
  'syms': { 'grokking.helpers': { 'grokking.helpers.animate_lines': ('helpers.html#animate_lines', 'grokking/helpers.py'),
                                  'grokking.helpers.animate_multi_lines': ('helpers.html#animate_multi_lines', 'grokking/helpers.py'),
                                  'grokking.helpers.animate_scatter': ('helpers.html#animate_scatter', 'grokking/helpers.py'),
                                  'grokking.helpers.cos': ('helpers.html#cos', 'grokking/helpers.py'),
                                  'grokking.helpers.cpu_aware_load_at_root': ('helpers.html#cpu_aware_load_at_root', 'grokking/helpers.py'),
                                  'grokking.helpers.cross_entropy_high_precision': ( 'helpers.html#cross_entropy_high_precision',
                                                                                     'grokking/helpers.py'),
                                  'grokking.helpers.cuda_memory': ('helpers.html#cuda_memory', 'grokking/helpers.py'),
                                  'grokking.helpers.download_model_from_drive': ( 'helpers.html#download_model_from_drive',
                                                                                  'grokking/helpers.py'),
                                  'grokking.helpers.extract_freq_2d': ('helpers.html#extract_freq_2d', 'grokking/helpers.py'),
                                  'grokking.helpers.full_loss': ('helpers.html#full_loss', 'grokking/helpers.py'),
                                  'grokking.helpers.get_cov': ('helpers.html#get_cov', 'grokking/helpers.py'),
                                  'grokking.helpers.imshow': ('helpers.html#imshow', 'grokking/helpers.py'),
                                  'grokking.helpers.imshow_fourier': ('helpers.html#imshow_fourier', 'grokking/helpers.py'),
                                  'grokking.helpers.install_dependencies': ('helpers.html#install_dependencies', 'grokking/helpers.py'),
                                  'grokking.helpers.is_close': ('helpers.html#is_close', 'grokking/helpers.py'),
                                  'grokking.helpers.is_inside_colab': ('helpers.html#is_inside_colab', 'grokking/helpers.py'),
                                  'grokking.helpers.line': ('helpers.html#line', 'grokking/helpers.py'),
                                  'grokking.helpers.line_marker': ('helpers.html#line_marker', 'grokking/helpers.py'),
                                  'grokking.helpers.lines': ('helpers.html#lines', 'grokking/helpers.py'),
                                  'grokking.helpers.load_5_digit_addition_finite': ( 'helpers.html#load_5_digit_addition_finite',
                                                                                     'grokking/helpers.py'),
                                  'grokking.helpers.load_5_digit_addition_infinite': ( 'helpers.html#load_5_digit_addition_infinite',
                                                                                       'grokking/helpers.py'),
                                  'grokking.helpers.load_finite_data_losses': ( 'helpers.html#load_finite_data_losses',
                                                                                'grokking/helpers.py'),
                                  'grokking.helpers.load_induction_head_finite': ( 'helpers.html#load_induction_head_finite',
                                                                                   'grokking/helpers.py'),
                                  'grokking.helpers.load_induction_head_infinite': ( 'helpers.html#load_induction_head_infinite',
                                                                                     'grokking/helpers.py'),
                                  'grokking.helpers.load_infinite_data_losses': ( 'helpers.html#load_infinite_data_losses',
                                                                                  'grokking/helpers.py'),
                                  'grokking.helpers.load_mod_addition_frac_train_sweep': ( 'helpers.html#load_mod_addition_frac_train_sweep',
                                                                                           'grokking/helpers.py'),
                                  'grokking.helpers.load_no_wd_width_scan': ('helpers.html#load_no_wd_width_scan', 'grokking/helpers.py'),
                                  'grokking.helpers.mod_div': ('helpers.html#mod_div', 'grokking/helpers.py'),
                                  'grokking.helpers.normalize': ('helpers.html#normalize', 'grokking/helpers.py'),
                                  'grokking.helpers.run_shell_command_as_python': ( 'helpers.html#run_shell_command_as_python',
                                                                                    'grokking/helpers.py'),
                                  'grokking.helpers.scatter': ('helpers.html#scatter', 'grokking/helpers.py'),
                                  'grokking.helpers.test_logits': ('helpers.html#test_logits', 'grokking/helpers.py'),
                                  'grokking.helpers.to_numpy': ('helpers.html#to_numpy', 'grokking/helpers.py'),
                                  'grokking.helpers.unflatten_first': ('helpers.html#unflatten_first', 'grokking/helpers.py')},
            'grokking.setup': {},
            'grokking.transformers': { 'grokking.transformers.Attention': ('transformer.html#attention', 'grokking/transformers.py'),
                                       'grokking.transformers.Attention.__init__': ( 'transformer.html#__init__',
                                                                                     'grokking/transformers.py'),
                                       'grokking.transformers.Attention.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.Embed': ('transformer.html#embed', 'grokking/transformers.py'),
                                       'grokking.transformers.Embed.__init__': ('transformer.html#__init__', 'grokking/transformers.py'),
                                       'grokking.transformers.Embed.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint': ('transformer.html#hookpoint', 'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint.__init__': ( 'transformer.html#__init__',
                                                                                     'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint.add_hook': ( 'transformer.html#add_hook',
                                                                                     'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint.give_name': ( 'transformer.html#give_name',
                                                                                      'grokking/transformers.py'),
                                       'grokking.transformers.HookPoint.remove_hooks': ( 'transformer.html#remove_hooks',
                                                                                         'grokking/transformers.py'),
                                       'grokking.transformers.LayerNorm': ('transformer.html#layernorm', 'grokking/transformers.py'),
                                       'grokking.transformers.LayerNorm.__init__': ( 'transformer.html#__init__',
                                                                                     'grokking/transformers.py'),
                                       'grokking.transformers.LayerNorm.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.MLP': ('transformer.html#mlp', 'grokking/transformers.py'),
                                       'grokking.transformers.MLP.__init__': ('transformer.html#__init__', 'grokking/transformers.py'),
                                       'grokking.transformers.MLP.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.PosEmbed': ('transformer.html#posembed', 'grokking/transformers.py'),
                                       'grokking.transformers.PosEmbed.__init__': ('transformer.html#__init__', 'grokking/transformers.py'),
                                       'grokking.transformers.PosEmbed.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.Transformer': ('transformer.html#transformer', 'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.__init__': ( 'transformer.html#__init__',
                                                                                       'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.cache_all': ( 'transformer.html#cache_all',
                                                                                        'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.forward': ( 'transformer.html#forward',
                                                                                      'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.hook_points': ( 'transformer.html#hook_points',
                                                                                          'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.remove_all_hooks': ( 'transformer.html#remove_all_hooks',
                                                                                               'grokking/transformers.py'),
                                       'grokking.transformers.Transformer.set_use_cache': ( 'transformer.html#set_use_cache',
                                                                                            'grokking/transformers.py'),
                                       'grokking.transformers.TransformerBlock': ( 'transformer.html#transformerblock',
                                                                                   'grokking/transformers.py'),
                                       'grokking.transformers.TransformerBlock.__init__': ( 'transformer.html#__init__',
                                                                                            'grokking/transformers.py'),
                                       'grokking.transformers.TransformerBlock.forward': ( 'transformer.html#forward',
                                                                                           'grokking/transformers.py'),
                                       'grokking.transformers.Unembed': ('transformer.html#unembed', 'grokking/transformers.py'),
                                       'grokking.transformers.Unembed.__init__': ('transformer.html#__init__', 'grokking/transformers.py'),
                                       'grokking.transformers.Unembed.forward': ('transformer.html#forward', 'grokking/transformers.py'),
                                       'grokking.transformers.gen_train_test': ( 'transformer.html#gen_train_test',
                                                                                 'grokking/transformers.py'),
                                       'grokking.transformers.train_model': ('transformer.html#train_model', 'grokking/transformers.py')}}}